# Copyright Keysight Technologies, Inc. 2013-2014
# All rights reserved.
# The contents of this file are not to be modified by the user
# unless directed to do so by Keysight Technologies, Inc.

package provide AgHwSetup 1.0;
package require AgGeneric

## \file AgHwSetup.tcl
# \brief This file defines functions for Hardware Setup.

## This function sets the default values of the DataGenerator based on its capabilities.
#<b>  Example : </b>
#<pre>        CONFIGURE_CONNECTION</pre>

proc CONFIGURE_CONNECTION {} { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
    log "No active session."
    return 
  }
  
  source $::TclDefineLoc
  source $::ExSetupPath//DllAttrDefine.tcl
  source $::ExSetupPath//DmeAttrDefine.tcl
  source $::ExSetupPath//MphyAttrDefine.tcl
  source $::ExSetupPath//NwAttrDefine.tcl
  source $::ExSetupPath//PaAttrDefine.tcl
  source $::ExSetupPath//TlAttrDefine.tcl
  
#  InitDataGeneratorSetting

  #puts "Resetting blocks ... \n" 
#  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_resetBitStuffer 0 1
#  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_resetSyncTimeStamp 0 1
#  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_resetSyncDrus 0 1

#  after 200
# GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_resetBitStuffer 0 0
# GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_resetSyncTimeStamp 0 0
# GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_resetSyncDrus 0 0

  #puts "Setting attributes for linkup ..."
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_burstIdleTime 0 100 
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_sync 0 0xff    
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_burstAutoMode 0 1 
  
  InitDefaultRegisterSetting
}

proc InitDataGeneratorSetting { } { 

  source $::TclDefineLoc
#  puts "PROGRAMMING OF Device capability of DataGenerator 0"
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_INIT_LINE_STATE_0 0 0x00
# GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_HOLD_LINE_STATE 0 0xFF
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_LOAD_LINE_STATE 0 0xFF
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_LOAD_LINE_STATE 0 0x00
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_IGNORE_HIBERN8 0 0x01   
}

proc InitDefaultRegisterSetting { } { 

  source $::TclDefineLoc
  set PA_TActivate 0x15A8
  set PA_TxTrailingClocks 0x1564
  set TX_LS_PREPARE_LENGTH 0x002A
  set linkupTimoeut 0x80a
  set linkupTimoeut2 0x80b
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_DME_linkStartupTime 1 0xffff 0xffff
  DME_SET.req $PA_TActivate 0x0 0x00FF
  DME_SET.req $PA_TxTrailingClocks 0x0 0xFF
  DME_SET.req $TX_LS_PREPARE_LENGTH 0x0 0xFF

}

set InvertHexValue(0x0) "1111" 
set InvertHexValue(0x1) "1110" 
set InvertHexValue(0x2) "1101" 
set InvertHexValue(0x3) "1100" 
set InvertHexValue(0x4) "1011" 
set InvertHexValue(0x5) "1010" 
set InvertHexValue(0x6) "1001" 
set InvertHexValue(0x7) "1000" 
set InvertHexValue(0x8) "0111" 
set InvertHexValue(0x9) "0110" 
set InvertHexValue(0xA) "0101" 
set InvertHexValue(0xB) "0100" 
set InvertHexValue(0xC) "0011" 
set InvertHexValue(0xD) "0010" 
set InvertHexValue(0xE) "0001" 
set InvertHexValue(0xF) "0000"

## This function inverts the polarity of transmit lanes.
    # @param[in] InvertLane \n Input type   : Hex Value\n
# Range         : [0x0 -0xF], LSB set to 1 indicates invert polarity of TX Lane0, MSB set to 1 indicates invert polarity of TX Lane3.\n
    # Description   : Specifies if polarity of a lane has to be inverted \n
#
#<b>  Example : </b>
#<pre>        INVERT_TX_POLARITY 0x1</pre>

proc INVERT_TX_POLARITY {InvertLane} { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
    log "No active session."
    return 
  }
  
  source $::TclDefineLoc
  source "$::ExSetupPath//MphyAttrDefine.tcl"
  log "Inverting Tx polarity ...\n"
	
  # Get Paddle board version
  set result [ agt_matrix::GetPaddleBoardRev $::agt_lahandle ] 
  
  if { $result == 0}  { 
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_txHsDataInvPol 0 $InvertLane
  } elseif { $result == 4 } {
   	set InvertLane $::InvertHexValue($InvertLane) 
	GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_txHsDataInvPol 0 $InvertLane
  }
  puts "result : $result"
	
  #GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_txHsDataInvPol 0 $InvertLane
   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_txCalInvPol 0 $InvertLane
   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_txLsDataInvPol 0 $InvertLane 
   
  WaitForTime 100
}


## This function inverts the polarity of receiver lanes.
    # @param[in] InvertLane \n Input type   : Hex Value\n
# Range         : [0x0 -0xF], LSB set to 1 indicates invert polarity of RX Lane0, MSB set to 1 indicates invert polarity of RX Lane3.\n
    # Description   : Specifies if polarity of a lane has to be inverted \n
#
#<b>  Example : </b>
#<pre>        INVERT_RX_POLARITY 0x1</pre>

proc INVERT_RX_POLARITY {InvertLane} { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
    log "No active session."
    return 
  }
  
  source $::TclDefineLoc
  log "Inverting Rx polarity ...\n"
  GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_rxInvPol 0 $InvertLane
}


## This function is used to Reset the DUT.For this connect a SMA connector from TRIG-OUT on the module to DUT's HW RESET port.
    # Description   : HW reset of DUT\n
#
#<b>  Example : </b>
#<pre>        HW_RESET </pre>

proc HW_RESET { { timeinterval 10000 } } { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
    log "No active session."
    return 
  }
  
  source $::TclDefineLoc
  puts "Asserting HW reset to DUT ..."

  if { $timeinterval != 0xC0 } { 
    set timeinterval [ expr { int ( $timeinterval ) } ]

    if { $timeinterval > 0 } { 
      set result [ expr { $timeinterval / 5 } ]
      set remindvalue [ expr { $timeinterval % 5 } ]
      if { $remindvalue != 0} { 
      set result [ expr { $result + 1 } ]
      }
    } else {
      set result 0
    }
  } else { 
      set result $timeinterval
  }

#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_trigOutTime 0 $result
    
#   GenericFunctions::writeGeneric 8 0x2400d 0 0x5
#   GenericFunctions::writeGeneric 8 0x2401b 0 0x2

#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x1
#   GenericFunctions::readGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x1
#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x0
#   GenericFunctions::readGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x1

#The default reset fails when 2 blades are used use this one.
  GenericFunctions::writeGeneric 8 0x2400d 0 0x5
  GenericFunctions::writeGeneric 8 0x2401b 0 0x0
  GenericFunctions::readGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x1
  after 100
  GenericFunctions::writeGeneric 8 0x2401b 0 0xf
  GenericFunctions::readGeneric $::ACR_CPU_LOWER $reg_enableTrigOut 0 0x1

  #give the device 10ms to recover
  after [expr $timeinterval/1000]
  #after 1000
  puts " "
}

  ## This function is used to set the clock rate series.The user can select if Rate Series A or B has to be used.The rate series cannot be changed dynamically using PACP_PWR_req. This has to set before doing other operations.
   # @param[in] clockrateseries \n Input type   : Enumeration\n
    # Range   : A, B.\n
    # Description   : set clock rate series .\n
    
    #<b>  Example : </b>
    #<pre>        SET_RATE_SERIES A </pre>
    
proc SET_RATE_SERIES { clockrateseries } { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
  log "No active session."
  return 
  }

  set clockrateseries [string toupper $clockrateseries ]
  
  if { $clockrateseries != "A"  && $clockrateseries != "B" } { 
    log "Invalid rate series."
    return 0
  }
  source $::TclDefineLoc
  source "$::ExSetupPath//PaAttrDefine.tcl"
#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_DME_enable 0 0x0
#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_DME_reset 0 0x1
#   after 1000
#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_DME_reset 0 0x0
#   GenericFunctions::writeGeneric $::ACR_CPU_LOWER $reg_SW_DME_enable 0 0x0
    
  if { $clockrateseries == "A" } { 
    log "Configuring Clock Rate Series A..."
    set result [ agt_matrix::SetClockRateSeries $::agt_lahandle A ] 
#     WaitForTime 2000
#     set result [ agt_matrix::SetClockRateSeries $::agt_lahandle B ] 
#     WaitForTime 2000
#     set result [ agt_matrix::SetClockRateSeries $::agt_lahandle A ] 
#     WaitForTime 2000
    DME_SET.req $PA_HSSeries 0x0 0x0001
  } 

  if { $clockrateseries == "B" } { 
    log "Configuring Clock Rate Series B..."
    set result [ agt_matrix::SetClockRateSeries $::agt_lahandle B ] 
#     WaitForTime 2000
#     set result [ agt_matrix::SetClockRateSeries $::agt_lahandle A ] 
#     WaitForTime 2000
#     set result [ agt_matrix::SetClockRateSeries $::agt_lahandle B ] 
#     WaitForTime 2000
    DME_SET.req $PA_HSSeries 0x0 0x0002
  }

}
    
  ## This function is used to select the clock source.The user can choose if internal or external clock source is to be used.
   # @param[in] clocksource \n Input type   : Enumeration\n
    # Range   : External, Internal.\n
    # Description   : set clock condition .\n
#<b>  Example : </b>
    #<pre>        SET_CLOCK_SOURCE External </pre>
    
proc SET_CLOCK_SOURCE { clocksource } { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
  log "No active session."
  return 
  }

  set clocksource [string toupper $clocksource ]

  if { $clocksource == "EXTERNAL" } { 
    log "Configuring External Clock Source ..."
    set result [ agt_matrix::SetClockRateConditioner $::agt_lahandle "TX" "EXT1" "26" "A"  ] 
  } 
  if { $clocksource == "INTERNAL" } { 
    log "Configuring Internal Clock Source ..."
    set result [ agt_matrix::SetClockRateConditioner $::agt_lahandle "TX" "OSC30" "30" "A"  ] 
  } 

  if { $clocksource != "EXTERNAL"  && $clocksource != "INTERNAL" } { 
    log "Invalid Clock Source.\n"
  }

  if { $result != "" } {
    log "$result \n"
  } else { 
    log "Configured Clock Source ...\n"
  }
}

  ## This function is used to set Logical to Physical Lane mapping.
   # @param[in] PhysicalLanes \n Input type   : 16 bit value in hex\n
    # Range   : 0000 - FFFF\n
    # Description   :\n
        #TX Lanes:\n
        # Bits [1:0]  : Physical Lane of Logical Lane 0.\n
        # Bits [3:2]  : Physical Lane of Logical Lane 1.\n
        # Bits [5:4]  : Physical Lane of Logical Lane 2.\n
        # Bits [7:6]  : Physical Lane of Logical Lane 3.\n
        #RX Lanes:\n
        # Bits [9:8]  : Physical Lane of Logical Lane 0.\n
        # Bits [11:10]: Physical Lane of Logical Lane 1.\n
        # Bits [13:12]: Physical Lane of Logical Lane 2.\n
        # Bits [15:14]: Physical Lane of Logical Lane 3.\n
        #\n
#<b>  Example : </b>
    #<pre>   SET_LANE_MAPPING   0xe4e4 </pre>
    
proc SET_LANE_MAPPING { PhysicalLanes } { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
  log "No active session."
  return 
  }

  source "$::ExSetupPath//PaAttrDefine.tcl"
  log "Setting Physical to Logical Lane Mapping ...\n"
  DME_SET.req $PA_LogicalLaneMap 0 $PhysicalLanes
  WaitForTime 100
}

  ## This function is used to set the the changing between large and small amplitude.
   # @param[in] ampType \n Input type   : Enumeration\n
    # Range   : LA, SA.\n
    # Description   : set voltage amplitude .\n
    
    #<b>  Example : </b>
    #<pre>        CHANGE_VOLTAGE LA </pre>
    
proc CHANGE_VOLTAGE { ampType } { 

  set isactsession [ IsValidSession ]
  if { $isactsession == 0 } {
  log "No active session."
  return 
  }

  set ampType [string toupper $ampType ]
  if { $ampType == "LA" } { 
    set result [ agt_matrix::SetTxAmplitude $::agt_lahandle all 400] 
    set result [ agt_matrix::SetTxCmVoltage $::agt_lahandle 210 400 ] 
  log "------------------------------------------------------------"

 log "Info: Setting Large Amplitude voltage ...\n"

    
  } elseif { $ampType == "SA" } { 
    set result [ agt_matrix::SetTxAmplitude $::agt_lahandle all 230] 
    set result [ agt_matrix::SetTxCmVoltage $::agt_lahandle 135 230 ] 
  log "------------------------------------------------------------"

     log "\Info: Setting Small Amplitude voltage ...\n"

  } else { 
  log "------------------------------------------------------------"

      log "Error: Invalid Voltage Type.\n"
  }
}
